ImmersiveHUDHider = ImmersiveHUDHider or {}
ImmersiveHUDHider.name = "ImmersiveHUDHider"
ImmersiveHUDHider.version = "1.3"

-- AUI Unit frames ID list
local AUIUnitFramesToHide = {}

local hideChatChoices = {[1] = "Don't Hide Chat", [2] = "Hide Only Background (keep icons)", [3] = "Fully Hide Chat"}
local hideCrosshairChoices = {
    [1] = "Don't Hide Crosshair (Pick me for compatibility with Combat Reticle addon)", [2] = "Hide Unless in Combat", [3] = "Always Hide Crosshair"
}

local function hideAUIUnitFrames()
    local gCurrentTemplateData = AUI.UnitFrames.GetActiveTemplates()
    for _, templateData in pairs(gCurrentTemplateData) do
        for frameType, data in pairs(templateData.frameData) do
            control = data.control
            for _, templateData in pairs(gCurrentTemplateData) do
                for frameType, data in pairs(templateData.frameData) do
                    control = data.control

                    local timelastrun = 0
                    control:SetHandler(
                        "OnUpdate", function(self, timerun)
                            if (timerun - timelastrun) >= 0 then
                                timelastrun = timerun
                                for k, v in pairs(AUIUnitFramesToHide) do
                                    AUI.UnitFrames.HideFrame(v)
                                end
                            end
                        end
                    )

                    break
                end
            end
            break
        end
    end
end

-- Default values for settings menu
ImmersiveHUDHider.defaultSettingsMenu = {
    hideInteractableGlow = false, hideTargetGlow = false, hideQuestBestowerIndicator = false, hideGroupIndicators = true, hideFollowIndicator = true,
    hideAllianceIndicators = true, hideResurrectIndicator = true, hideAllHealthBars = false, hideAllNamePlates = true, hideChatBubbles = true,
    hideActionBar = true, hideAttributeBar = true, hideBuffs = true, hideScrollingCombatText = true, hideLootLog = true, hideCompass = true,
    hideMap = true, hideQuestTracker = true, hideGroupUnitFrames = true, hideBossBar = true, hideTargetInfo = true, hideChat = hideChatChoices[3]
}

-- Keep track of default options for in-game HUD settings.
-- Returns previous vanilla settings back to these on disable
ImmersiveHUDHider.default = {
    isInterfaceEventHandler = 0, ImmersiveHUDHider_isUIHidden = false,
    defaultInteractableGlow = GetSetting(SETTING_TYPE_IN_WORLD, IN_WORLD_UI_SETTING_INTERACTABLE_GLOW_ENABLED),
    defaultTargetGlow = GetSetting(SETTING_TYPE_IN_WORLD, IN_WORLD_UI_SETTING_TARGET_GLOW_ENABLED),
    defaultQuestBestowerIndicator = GetSetting(SETTING_TYPE_UI, UI_SETTING_SHOW_QUEST_BESTOWER_INDICATORS),
    defaultGroupIndicators = GetSetting(SETTING_TYPE_NAMEPLATES, NAMEPLATE_TYPE_GROUP_INDICATORS),
    defaultFollowIndicator = GetSetting(SETTING_TYPE_NAMEPLATES, NAMEPLATE_TYPE_FOLLOWER_INDICATORS),
    defaultAllianceIndicators = GetSetting(SETTING_TYPE_NAMEPLATES, NAMEPLATE_TYPE_ALLIANCE_INDICATORS),
    defaultResurrectIndicator = GetSetting(SETTING_TYPE_NAMEPLATES, NAMEPLATE_TYPE_RESURRECT_INDICATORS),
    defaultAllHealthBars = GetSetting(SETTING_TYPE_NAMEPLATES, NAMEPLATE_TYPE_ALL_HEALTHBARS),
    defaultAllNamePlates = GetSetting(SETTING_TYPE_NAMEPLATES, NAMEPLATE_TYPE_ALL_NAMEPLATES),
    defaultChatBubbles = GetSetting(SETTING_TYPE_CHAT_BUBBLE, CHAT_BUBBLE_SETTING_ENABLED),
    defaultChatBubbleSpeed = GetSetting(SETTING_TYPE_CHAT_BUBBLE, CHAT_BUBBLE_SETTING_SPEED_MODIFIER),
    defaultActionBar = GetSetting(SETTING_TYPE_UI, UI_SETTING_SHOW_ACTION_BAR),
    defaultAttributeBar = GetSetting(SETTING_TYPE_UI, UI_SETTING_SHOW_RESOURCE_BARS),
    defaultBuffs = GetSetting(SETTING_TYPE_BUFFS, BUFFS_SETTING_ALL_ENABLED),
    defaultScrollingCombatText = GetSetting(SETTING_TYPE_COMBAT, COMBAT_SETTING_SCROLLING_COMBAT_TEXT_ENABLED),
    defaultLootLog = GetSetting(SETTING_TYPE_LOOT, LOOT_SETTING_LOOT_HISTORY), hideInteractableGlow = false, hideTargetGlow = false,
    hideQuestBestowerIndicator = false, hideGroupIndicators = true, hideFollowIndicator = true, hideAllianceIndicators = true,
    hideResurrectIndicator = true, hideAllHealthBars = false, hideAllNamePlates = true, hideChatBubbles = true, hideActionBar = true,
    hideAttributeBar = true, hideBuffs = true, hideScrollingCombatText = true, hideLootLog = true, hideCompass = true, hideMap = true,
    hideQuestTracker = true, hideGroupUnitFrames = true, hideBossBar = true, hideTargetInfo = true, hideChat = hideChatChoices[3],
    hideCrosshair = hideCrosshairChoices[2]
}

-- Save default values to file
function ImmersiveHUDHider.saveDefaultGameSettings()

    -- Save default in-game settings to file.
    -- Used to revert settings to what they previously were
    if ImmersiveHUDHider.savedVariables.ImmersiveHUDHider_isUIHidden == false then
        ImmersiveHUDHider.savedVariables.defaultInteractableGlow = GetSetting(SETTING_TYPE_IN_WORLD, IN_WORLD_UI_SETTING_INTERACTABLE_GLOW_ENABLED)
        ImmersiveHUDHider.savedVariables.defaultTargetGlow = GetSetting(SETTING_TYPE_IN_WORLD, IN_WORLD_UI_SETTING_TARGET_GLOW_ENABLED)
        ImmersiveHUDHider.savedVariables.defaultQuestBestowerIndicator = GetSetting(SETTING_TYPE_UI, UI_SETTING_SHOW_QUEST_BESTOWER_INDICATORS)
        ImmersiveHUDHider.savedVariables.defaultGroupIndicators = GetSetting(SETTING_TYPE_NAMEPLATES, NAMEPLATE_TYPE_GROUP_INDICATORS)
        ImmersiveHUDHider.savedVariables.defaultFollowIndicator = GetSetting(SETTING_TYPE_NAMEPLATES, NAMEPLATE_TYPE_FOLLOWER_INDICATORS)
        ImmersiveHUDHider.savedVariables.defaultAllianceIndicators = GetSetting(SETTING_TYPE_NAMEPLATES, NAMEPLATE_TYPE_ALLIANCE_INDICATORS)
        ImmersiveHUDHider.savedVariables.defaultResurrectIndicator = GetSetting(SETTING_TYPE_NAMEPLATES, NAMEPLATE_TYPE_RESURRECT_INDICATORS)
        ImmersiveHUDHider.savedVariables.defaultAllHealthBars = GetSetting(SETTING_TYPE_NAMEPLATES, NAMEPLATE_TYPE_ALL_HEALTHBARS)
        ImmersiveHUDHider.savedVariables.defaultAllNamePlates = GetSetting(SETTING_TYPE_NAMEPLATES, NAMEPLATE_TYPE_ALL_NAMEPLATES)
        ImmersiveHUDHider.savedVariables.defaultChatBubbles = GetSetting(SETTING_TYPE_CHAT_BUBBLE, CHAT_BUBBLE_SETTING_ENABLED)
        ImmersiveHUDHider.savedVariables.defaultChatBubbleSpeed = GetSetting(SETTING_TYPE_CHAT_BUBBLE, CHAT_BUBBLE_SETTING_SPEED_MODIFIER)
        ImmersiveHUDHider.savedVariables.defaultActionBar = GetSetting(SETTING_TYPE_UI, UI_SETTING_SHOW_ACTION_BAR)
        ImmersiveHUDHider.savedVariables.defaultAttributeBar = GetSetting(SETTING_TYPE_UI, UI_SETTING_SHOW_RESOURCE_BARS)
        ImmersiveHUDHider.savedVariables.defaultBuffs = GetSetting(SETTING_TYPE_BUFFS, BUFFS_SETTING_ALL_ENABLED)
        ImmersiveHUDHider.savedVariables.defaultScrollingCombatText = GetSetting(SETTING_TYPE_COMBAT, COMBAT_SETTING_SCROLLING_COMBAT_TEXT_ENABLED)
        ImmersiveHUDHider.savedVariables.defaultLootLog = GetSetting(SETTING_TYPE_LOOT, LOOT_SETTING_LOOT_HISTORY)
        ImmersiveHUDHider.savedVariables.defaultQuestTracker = GetSetting(SETTING_TYPE_UI, UI_SETTING_SHOW_QUEST_TRACKER)
    end

end

function ImmersiveHUDHider.interfaceEventHandlerToggle()
    if ImmersiveHUDHider.savedVariables.isInterfaceEventHandler == 1 then
        -- EVENT_MANAGER:UnregisterForEvent(ImmersiveHUDHider.name, EVENT_INTERFACE_SETTING_CHANGED)
    else
        -- EVENT_MANAGER:RegisterForEvent(ImmersiveHUDHider.name, EVENT_INTERFACE_SETTING_CHANGED, ImmersiveHUDHider.saveDefaultGameSettings)
    end
end

function ImmersiveHUDHider.isInCombat(_, inCombat)
    if inCombat == nil then
        inCombat = IsUnitInCombat("player")
    end

    if inCombat then
        ZO_ReticleContainerReticle:SetHidden(false)
        local timelastrun = 0
        ZO_ReticleContainerReticle:SetHandler(
            "OnUpdate", function(self, timerun)
                if (timerun - timelastrun) >= 0 then
                    timelastrun = timerun
                    ZO_ReticleContainerReticle:SetHidden(false)
                end
            end
        )
    else
        local timelastrun = 0
        ZO_ReticleContainerReticle:SetHandler(
            "OnUpdate", function(self, timerun)
                if (timerun - timelastrun) >= 0 then
                    timelastrun = timerun
                    ZO_ReticleContainerReticle:SetHidden(true)
                end
            end
        )
    end

    Options_Audio_MusicVolumeSlider:SetEnabled(not inCombat) -- Disable changes to the official audio setting slider when in combat
end

function hideCrosshair(boolean)
    -- Show only in combat
    if (ImmersiveHUDHider.savedVariables.hideCrosshair == hideCrosshairChoices[2]) then
        if (boolean) then
            EVENT_MANAGER:RegisterForEvent("CrosshairInCombat", EVENT_PLAYER_COMBAT_STATE, ImmersiveHUDHider.isInCombat)
            ZO_ReticleContainerReticle:SetHidden(true)
        else
            ZO_ReticleContainerReticle:SetHidden(false)
        end
        -- Always Hide
    elseif (ImmersiveHUDHider.savedVariables.hideCrosshair == hideCrosshairChoices[3]) then
        if (boolean) then
            local timelastrun = 0
            ZO_ReticleContainerReticle:SetHandler(
                "OnUpdate", function(self, timerun)
                    if (timerun - timelastrun) >= 0 then
                        timelastrun = timerun
                        ZO_ReticleContainerReticle:SetHidden(true)
                    end
                end
            )

        else
            ZO_ReticleContainerReticle:SetHidden(false)
        end
        -- Always Show (keep default)
    else

    end
end

function hideChat(boolean)
    -- Hide only background
    if (ImmersiveHUDHider.savedVariables.hideChat == hideChatChoices[2]) then
        if (boolean) then
            -- ZO_ChatWindow:SetHidden(true)
            -- ZO_ChatWindow:SetAlpha(0)
            ZO_ChatWindowBg:SetAlpha(0)
            -- ZO_ChatWindowDivider:SetAlpha(0)
            ZO_ChatWindowMinBar:SetAlpha(0)
            -- ZO_ChatWindowNotifications:SetAlpha(0)
        else
            -- ZO_ChatWindow:SetHidden(false)
            -- ZO_ChatWindow:SetAlpha(100)
            ZO_ChatWindowBg:SetAlpha(100)
            -- ZO_ChatWindowDivider:SetAlpha(100)
            ZO_ChatWindowMinBar:SetAlpha(100)
            CHAT_SYSTEM:SetChannel(CHAT_SYSTEM.currentChannel)
            -- ZO_ChatWindowNotifications:SetAlpha(100)
        end
        -- Fully hide chat
    elseif (ImmersiveHUDHider.savedVariables.hideChat == hideChatChoices[3]) then
        if (boolean) then
            ZO_ChatWindow:SetHidden(true)
            ZO_ChatWindow:SetAlpha(0)
            ZO_ChatWindowBg:SetAlpha(0)
            ZO_ChatWindowDivider:SetAlpha(0)
            ZO_ChatWindowMinBar:SetAlpha(0)
            ZO_ChatWindowNotifications:SetAlpha(0)

            local timelastrun = 0
            ZO_ChatWindow:SetHandler(
                "OnUpdate", function(self, timerun)
                    if (timerun - timelastrun) >= 0 then
                        timelastrun = timerun
                        ZO_ChatWindow:SetHidden(true)
                        ZO_ChatWindow:SetAlpha(0)
                        ZO_ChatWindowBg:SetAlpha(0)
                        ZO_ChatWindowDivider:SetAlpha(0)
                        ZO_ChatWindowMinBar:SetAlpha(0)
                        ZO_ChatWindowNotifications:SetAlpha(0)
                    end
                end
            )
        else
            ZO_ChatWindow:SetHidden(false)
            CHAT_SYSTEM:SetChannel(CHAT_SYSTEM.currentChannel)
            ZO_ChatWindow:SetAlpha(100)
            ZO_ChatWindowBg:SetAlpha(100)
            ZO_ChatWindowDivider:SetAlpha(100)
            ZO_ChatWindowMinBar:SetAlpha(100)
            ZO_ChatWindowNotifications:SetAlpha(100)

            ZO_ChatWindow:SetHandler(
                "OnUpdate", function(self, timerun)
                end
            )
        end
        -- Show chat
    else
        if (boolean) then
            -- ZO_ChatWindow:SetHidden(true)
            -- ZO_ChatWindow:SetAlpha(0)
            -- ZO_ChatWindowBg:SetAlpha(0)
            -- ZO_ChatWindowDivider:SetAlpha(0)
            -- ZO_ChatWindowMinBar:SetAlpha(0)
            -- ZO_ChatWindowNotifications:SetAlpha(0)
        else
            -- ZO_ChatWindow:SetHidden(false)
            -- ZO_ChatWindow:SetAlpha(100)
            -- ZO_ChatWindowBg:SetAlpha(100)
            -- ZO_ChatWindowDivider:SetAlpha(100)
            -- ZO_ChatWindowMinBar:SetAlpha(100)
            -- ZO_ChatWindowNotifications:SetAlpha(100)
        end
    end
end

function hideInteractableGlow(boolean)
    if (ImmersiveHUDHider.savedVariables.hideInteractableGlow) then
        if (boolean) then
            SetSetting(SETTING_TYPE_IN_WORLD, IN_WORLD_UI_SETTING_INTERACTABLE_GLOW_ENABLED, "false", DO_NOT_SAVE_TO_PERSISTED_DATA)
        else
            SetSetting(
                SETTING_TYPE_IN_WORLD, IN_WORLD_UI_SETTING_INTERACTABLE_GLOW_ENABLED, ImmersiveHUDHider.savedVariables.defaultInteractableGlow,
                DO_NOT_SAVE_TO_PERSISTED_DATA
            )
        end
    end
end

function hideTargetGlow(boolean)
    if (ImmersiveHUDHider.savedVariables.hideTargetGlow) then
        if (boolean) then
            SetSetting(SETTING_TYPE_IN_WORLD, IN_WORLD_UI_SETTING_TARGET_GLOW_ENABLED, "false", DO_NOT_SAVE_TO_PERSISTED_DATA)
        else
            SetSetting(
                SETTING_TYPE_IN_WORLD, IN_WORLD_UI_SETTING_TARGET_GLOW_ENABLED, ImmersiveHUDHider.savedVariables.defaultTargetGlow,
                DO_NOT_SAVE_TO_PERSISTED_DATA
            )
        end
    end
end

function hideQuestBestowerIdicator(boolean)
    if (ImmersiveHUDHider.savedVariables.hideQuestBestowerIndicator) then
        if (boolean) then
            SetSetting(SETTING_TYPE_UI, UI_SETTING_SHOW_QUEST_BESTOWER_INDICATORS, "false", DO_NOT_SAVE_TO_PERSISTED_DATA)
        else
            SetSetting(
                SETTING_TYPE_UI, UI_SETTING_SHOW_QUEST_BESTOWER_INDICATORS, ImmersiveHUDHider.savedVariables.defaultQuestBestowerIndicator,
                DO_NOT_SAVE_TO_PERSISTED_DATA
            )
        end
    end
end

function hideGroupIndicators(boolean)
    if (ImmersiveHUDHider.savedVariables.hideGroupIndicators) then
        if (boolean) then
            SetSetting(SETTING_TYPE_NAMEPLATES, NAMEPLATE_TYPE_GROUP_INDICATORS, "false", DO_NOT_SAVE_TO_PERSISTED_DATA)
        else
            SetSetting(
                SETTING_TYPE_NAMEPLATES, NAMEPLATE_TYPE_GROUP_INDICATORS, ImmersiveHUDHider.savedVariables.defaultGroupIndicators,
                DO_NOT_SAVE_TO_PERSISTED_DATA
            )
        end

    end
end

function hideFollowIndicator(boolean)
    if (ImmersiveHUDHider.savedVariables.hideFollowIndicator) then
        if (boolean) then
            SetSetting(SETTING_TYPE_NAMEPLATES, NAMEPLATE_TYPE_FOLLOWER_INDICATORS, "false", DO_NOT_SAVE_TO_PERSISTED_DATA)
        else
            SetSetting(
                SETTING_TYPE_NAMEPLATES, NAMEPLATE_TYPE_FOLLOWER_INDICATORS, ImmersiveHUDHider.savedVariables.defaultFollowIndicator,
                DO_NOT_SAVE_TO_PERSISTED_DATA
            )
        end
    end
end

function hideAllianceIndicators(boolean)
    if (ImmersiveHUDHider.savedVariables.hideAllianceIndicators) then
        if (boolean) then
            SetSetting(SETTING_TYPE_NAMEPLATES, NAMEPLATE_TYPE_ALLIANCE_INDICATORS, NAMEPLATE_CHOICE_NEVER, DO_NOT_SAVE_TO_PERSISTED_DATA)
        else
            SetSetting(
                SETTING_TYPE_NAMEPLATES, NAMEPLATE_TYPE_ALLIANCE_INDICATORS, ImmersiveHUDHider.savedVariables.defaultAllianceIndicators,
                DO_NOT_SAVE_TO_PERSISTED_DATA
            )
        end
    end
end

function hideResurrectIndicator(boolean)
    if (ImmersiveHUDHider.savedVariables.hideResurrectIndicator) then
        if (boolean) then
            SetSetting(SETTING_TYPE_NAMEPLATES, NAMEPLATE_TYPE_RESURRECT_INDICATORS, "false", DO_NOT_SAVE_TO_PERSISTED_DATA)
        else
            SetSetting(
                SETTING_TYPE_NAMEPLATES, NAMEPLATE_TYPE_RESURRECT_INDICATORS, ImmersiveHUDHider.savedVariables.defaultResurrectIndicator,
                DO_NOT_SAVE_TO_PERSISTED_DATA
            )
        end
    end
end

function hideAllNamePlates(boolean)
    if (ImmersiveHUDHider.savedVariables.hideAllNamePlates) then
        if (boolean) then
            SetSetting(SETTING_TYPE_NAMEPLATES, NAMEPLATE_TYPE_ALL_NAMEPLATES, "false", DO_NOT_SAVE_TO_PERSISTED_DATA)
        else
            SetSetting(
                SETTING_TYPE_NAMEPLATES, NAMEPLATE_TYPE_ALL_NAMEPLATES, ImmersiveHUDHider.savedVariables.defaultAllNamePlates,
                DO_NOT_SAVE_TO_PERSISTED_DATA
            )
        end
    end
end

function hideChatBubbles(boolean)
    if (ImmersiveHUDHider.savedVariables.hideChatBubbles) then
        if (boolean) then
            SetSetting(SETTING_TYPE_CHAT_BUBBLE, CHAT_BUBBLE_SETTING_ENABLED, "false", DO_NOT_SAVE_TO_PERSISTED_DATA)
            SetSetting(SETTING_TYPE_CHAT_BUBBLE, CHAT_BUBBLE_SETTING_SPEED_MODIFIER, "0.80000001")
        else
            SetSetting(
                SETTING_TYPE_CHAT_BUBBLE, CHAT_BUBBLE_SETTING_ENABLED, ImmersiveHUDHider.savedVariables.defaultChatBubbles,
                DO_NOT_SAVE_TO_PERSISTED_DATA
            )
            SetSetting(SETTING_TYPE_CHAT_BUBBLE, CHAT_BUBBLE_SETTING_SPEED_MODIFIER, ImmersiveHUDHider.savedVariables.defaultChatBubbleSpeed)
        end

    end
end

function hideActionBar(boolean)
    if (ImmersiveHUDHider.savedVariables.hideActionBar) then
        if (boolean) then
            -- vanilla
            SetSetting(SETTING_TYPE_UI, UI_SETTING_SHOW_ACTION_BAR, "false", DO_NOT_SAVE_TO_PERSISTED_DATA)
            -- with GCD bar support
            if (GCDBar) then
                GCDBar.savedVars.hideFrame = true
                GCDBar.UI.setHudToggle(false)
                GCDBar.UI.gbFrame:SetHidden(true)
            end
        else
            SetSetting(SETTING_TYPE_UI, UI_SETTING_SHOW_ACTION_BAR, ImmersiveHUDHider.savedVariables.defaultActionBar, DO_NOT_SAVE_TO_PERSISTED_DATA)
            -- with GCD bar support
            if (GCDBar) then
                GCDBar.savedVars.hideFrame = false
                GCDBar.UI.setHudToggle(true)
                GCDBar.UI.gbFrame:SetHidden(false)
            end
        end
    end
end

function hideBuffs(boolean)
    if (ImmersiveHUDHider.savedVariables.hideBuffs == true) then
        if (boolean) then
            -- Vanilla
            SetSetting(SETTING_TYPE_BUFFS, BUFFS_SETTING_ALL_ENABLED, "false", DO_NOT_SAVE_TO_PERSISTED_DATA)

            -- AUI
            if (AUI) then
                if (AUI.Buffs.IsEnabled()) then

                    -- unitTag = AUI_TARGET_UNIT_TAG
                    -- effectType = BUFF_EFFECT_TYPE_DEBUFF
                    local _unitTag = "player"
                    local _effectType = BUFF_EFFECT_TYPE_BUFF
                    local activeBuffs = {}
                    if not activeBuffs[_unitTag] then
                        activeBuffs[_unitTag] = {}
                    end

                    if not activeBuffs[_unitTag][_effectType] then
                        activeBuffs[_unitTag][_effectType] = {}
                    end

                    for unitTag, unitData in pairs(activeBuffs) do
                        for effectType, buffControls in pairs(unitData) do
                            for _, control in pairs(buffControls) do
                                RemoveBuff(control.buffData.AbilityId, unitTag, effectType, false)
                                control:SetHidden(true)
                            end
                        end
                    end

                end
            end
        else
            SetSetting(SETTING_TYPE_BUFFS, BUFFS_SETTING_ALL_ENABLED, ImmersiveHUDHider.savedVariables.defaultBuffs, DO_NOT_SAVE_TO_PERSISTED_DATA)
        end
    end
end

function hideScrollingCombatText(boolean)
    if (ImmersiveHUDHider.savedVariables.hideScrollingCombatText == true) then
        if (boolean) then
            SetSetting(SETTING_TYPE_COMBAT, COMBAT_SETTING_SCROLLING_COMBAT_TEXT_ENABLED, "false", DO_NOT_SAVE_TO_PERSISTED_DATA)
        else
            SetSetting(
                SETTING_TYPE_COMBAT, COMBAT_SETTING_SCROLLING_COMBAT_TEXT_ENABLED, ImmersiveHUDHider.savedVariables.defaultScrollingCombatText,
                DO_NOT_SAVE_TO_PERSISTED_DATA
            )
        end
    end
end

function hideLootLog(boolean)
    if (ImmersiveHUDHider.savedVariables.hideLootLog == true) then
        if (boolean) then
            SetSetting(SETTING_TYPE_LOOT, LOOT_SETTING_LOOT_HISTORY, "false", DO_NOT_SAVE_TO_PERSISTED_DATA)

            if (LootDropGui) then
                LootDropGui:SetHidden(true)
            end

        else
            SetSetting(SETTING_TYPE_LOOT, LOOT_SETTING_LOOT_HISTORY, ImmersiveHUDHider.savedVariables.defaultLootLog, DO_NOT_SAVE_TO_PERSISTED_DATA)
            if (LootDropGui) then
                LootDropGui:SetHidden(false)
            end
        end
    end
end

function hideCompass(boolean)
    if (ImmersiveHUDHider.savedVariables.hideCompass) then
        if (boolean == true) then
            ZO_Compass:SetHidden(true)
            ZO_Compass:SetAlpha(0)
            ZO_CompassFrameCenter:SetHidden(true)
            ZO_CompassFrameLeft:SetHidden(true)
            ZO_CompassFrameRight:SetHidden(true)

            -- Hide compass after it has been updated
            -- Prevents ghosting after opening menu
            local timelastrun = 0
            ZO_CompassFrame:SetHandler(
                "OnUpdate", function(self, timerun)
                    if (timerun - timelastrun) >= 0 then
                        timelastrun = timerun
                        ZO_Compass:SetHidden(true)
                        ZO_BossBar:SetAlpha(0)
                        ZO_CompassFrame:SetHidden(true)
                    end
                end
            )
        else
            ZO_Compass:SetHidden(false)
            ZO_Compass:SetAlpha(100)
            ZO_CompassFrameCenter:SetHidden(false)
            ZO_CompassFrameLeft:SetHidden(false)
            ZO_CompassFrameRight:SetHidden(false)
            ZO_BossBar:SetAlpha(100)
            ZO_CompassFrame:SetHidden(false)

            -- Hide compass after it has been updated
            -- Prevents ghosting after opening menu
            local timelastrun = 0
            ZO_CompassFrame:SetHandler(
                "OnUpdate", function(self, timerun)

                end
            )
        end
    end
end

function hideMinimap(boolean)
    if (ImmersiveHUDHider.savedVariables.hideMap) then
        if (boolean == true) then
            if (AUI) then

                if (AUI.Minimap.IsEnabled()) then
                    AUI.Minimap.Hide()
                    AUI.Settings.Minimap.hidden = true
                end
            end
            if (FyrMM) then
                FyrMM.Visible = false
                FyrMM.AutoHidden = true
            end
            ZO_WorldMap:SetAlpha(0)
        else
            if (AUI) then
                if (AUI.Minimap.IsEnabled()) then
                    AUI.Minimap.Show()
                    AUI.Settings.Minimap.hidden = false
                end
            end
            if (FyrMM) then
                FyrMM.Visible = true
                FyrMM.AutoHidden = false
            end
            ZO_WorldMap:SetAlpha(100)
        end
    end
end

function hideQuestTracker(boolean)
    if (ImmersiveHUDHider.savedVariables.hideQuestTracker) then
        -- Vanilla
        if (boolean) then
            SetSetting(SETTING_TYPE_UI, UI_SETTING_SHOW_QUEST_TRACKER, "false")
        else
            SetSetting(SETTING_TYPE_UI, UI_SETTING_SHOW_QUEST_TRACKER, ImmersiveHUDHider.savedVariables.defaultQuestTracker)
        end

        -- AUI
        if (AUI_Questtracker) then
            if (boolean) then
                AUI_Questtracker:SetAlpha(0)
            else
                AUI_Questtracker:SetAlpha(100)
            end
        end

        -- Ravlox' Quest Tracker
        if (QUESTTRACKER) then
            if (boolean) then
                QuestTracker_ToggleWindow()
            else
                QUESTTRACKER.svCurrent.mainWindow.hideQuestWindow = false
                QUESTTRACKER.WINDOW_FRAGMENT:SetHiddenForReason("QuestTracker_UserSetting_Hidden", false)
            end
        end

        -- Addon for DailyAutoShare
        if (DasControl) then
            if (boolean) then
                local timelastrun = 0
                DasControl:SetHandler(
                    "OnUpdate", function(self, timerun)
                        if (timerun - timelastrun) >= 0 then
                            timelastrun = timerun
                            DasControl:SetHidden(true)
                        end
                    end
                )

            else
                DasControl:SetHandler(
                    "OnUpdate", function(self, timerun)
                    end
                )
                DasControl:SetHidden(false)
            end

        end

        -- Addon for Event Tracker 
        --if (EVT) then
        --    EVT.ToggleUI()
        --end
    end
end

function hideTargetInfo(boolean)
    if (ImmersiveHUDHider.savedVariables.hideTargetInfo) then
        if (boolean == true) then
            -- Default target frame
            EVENT_MANAGER:RegisterForEvent(
                "toggleTargetInfo", EVENT_RETICLE_TARGET_CHANGED, function()
                    if ZO_TargetUnitFramereticleover then
                        ZO_TargetUnitFramereticleover:SetHidden(true)
                    end
                end
            )
            -- AUI target frame
            if (AUI) then
                if (AUI.UnitFrames.Target.IsEnabled()) then
                    table.insert(AUIUnitFramesToHide, 201)
                    table.insert(AUIUnitFramesToHide, 202)
                    table.insert(AUIUnitFramesToHide, 211)
                    table.insert(AUIUnitFramesToHide, 212)
                    hideAUIUnitFrames()
                end
            end
        else
            -- Reset main target hud
            EVENT_MANAGER:UnregisterForEvent(
                "toggleTargetInfo", EVENT_RETICLE_TARGET_CHANGED, function()
                    if ZO_TargetUnitFramereticleover then
                        ZO_TargetUnitFramereticleover:SetHidden(false)
                    end
                end
            )

            -- Reset AUI target hud
            if (AUI) then
                if (AUI.UnitFrames.Target.IsEnabled()) then
                    AUIUnitFramesToHide = {}
                    hideAUIUnitFrames()
                    AUI.UnitFrames.UpdateUI()
                end
            end
        end
    end
end

function hideBossBar(boolean)
    if (ImmersiveHUDHider.savedVariables.hideBossBar) then
        if (boolean == true) then
            ZO_BossBar:SetAlpha(0)
        else
            ZO_BossBar:SetAlpha(100)
        end
    end
end

function hideAttributeBars(boolean)
    if (ImmersiveHUDHider.savedVariables.hideAttributeBar) then
        if (boolean == true) then
            -- Vanilla
            SetSetting(SETTING_TYPE_UI, UI_SETTING_SHOW_RESOURCE_BARS, "false", DO_NOT_SAVE_TO_PERSISTED_DATA)

            -- AUI
            if (AUI) then
                if (AUI.UnitFrames.Player.IsEnabled()) then
                    local gCurrentTemplateData = AUI.UnitFrames.GetActiveTemplates()
                    for _, templateData in pairs(gCurrentTemplateData) do
                        for frameType, data in pairs(templateData.frameData) do
                            control = data.control
                            for _, templateData in pairs(gCurrentTemplateData) do
                                for frameType, data in pairs(templateData.frameData) do
                                    control = data.control

                                    if (AUI.UnitFrames.IsPlayer(control.attributeId)) then
                                        table.insert(AUIUnitFramesToHide, 101)
                                        table.insert(AUIUnitFramesToHide, 102)
                                        table.insert(AUIUnitFramesToHide, 103)
                                        hideAUIUnitFrames()
                                    end

                                    break
                                end
                            end
                            break
                        end
                    end
                end
            end

        else
            -- Vanilla
            SetSetting(
                SETTING_TYPE_UI, UI_SETTING_SHOW_RESOURCE_BARS, ImmersiveHUDHider.savedVariables.defaultAttributeBar, DO_NOT_SAVE_TO_PERSISTED_DATA
            )

            -- AUI
            if (AUI) then
                if (AUI.UnitFrames.Player.IsEnabled()) then
                    AUIUnitFramesToHide = {}
                    hideAUIUnitFrames()
                    AUI.UnitFrames.UpdateUI()
                end
            end
        end
    end
end

function hideFloatingMarkers(boolean)
    if (ImmersiveHUDHider.savedVariables.hideQuestBestowerIndicator) then
        if (boolean == true) then
            SetFloatingMarkerGlobalAlpha(0)
        else
            SetFloatingMarkerGlobalAlpha(100)
        end
    end
end

function hideGroupFrames(boolean)
    if (ImmersiveHUDHider.savedVariables.hideGroupUnitFrames) then
        if (boolean) then
            -- Vanilla
            ZO_UnitFramesGroups:SetHidden(true)

            -- AUI
            if (AUI) then
                if (AUI.UnitFrames.Group.IsEnabled()) then
                    local gCurrentTemplateData = AUI.UnitFrames.GetActiveTemplates()
                    for _, templateData in pairs(gCurrentTemplateData) do
                        for frameType, data in pairs(templateData.frameData) do
                            control = data.control
                            for _, templateData in pairs(gCurrentTemplateData) do
                                for frameType, data in pairs(templateData.frameData) do
                                    control = data.control

                                    if (AUI.UnitFrames.IsGroupOrRaid(control.attributeId) or AUI.UnitFrames.IsMainCompanion(control.attributeId)) then
                                        table.insert(AUIUnitFramesToHide, 301)
                                        table.insert(AUIUnitFramesToHide, 302)
                                        table.insert(AUIUnitFramesToHide, 303)
                                        hideAUIUnitFrames()
                                    end

                                    break
                                end
                            end
                            break
                        end
                    end
                end
            end
        else
            -- Vanilla
            ZO_UnitFramesGroups:SetHidden(false)

            -- AUI
            if (AUI) then
                if (AUI.UnitFrames.Group.IsEnabled()) then
                    ZO_UnitFramesGroups:SetHidden(true)
                    AUIUnitFramesToHide = {}
                    hideAUIUnitFrames()
                end
            end
        end
    end

end

function hideNameplateHealthbars(boolean)
    if (ImmersiveHUDHider.savedVariables.hideAllHealthBars) then
        if (boolean) then
            SetSetting(SETTING_TYPE_NAMEPLATES, NAMEPLATE_TYPE_ALL_HEALTHBARS, "false", DO_NOT_SAVE_TO_PERSISTED_DATA)
        else
            SetSetting(SETTING_TYPE_NAMEPLATES, NAMEPLATE_TYPE_ALL_HEALTHBARS, ImmersiveHUDHider.savedVariables.defaultAllHealthBars, 1)
        end
    end
end

function hideHUD(boolean)
    hideInteractableGlow(boolean)
    hideTargetGlow(boolean)
    hideQuestBestowerIdicator(boolean)
    hideGroupIndicators(boolean)
    hideFollowIndicator(boolean)
    hideAllianceIndicators(boolean)
    hideResurrectIndicator(boolean)
    hideAllNamePlates(boolean)
    hideChatBubbles(boolean)
    hideActionBar(boolean)
    hideBuffs(boolean)
    hideScrollingCombatText(boolean)
    hideLootLog(boolean)
    hideCompass(boolean)
    hideMinimap(boolean)
    hideQuestTracker(boolean)
    hideTargetInfo(boolean)
    hideBossBar(boolean)
    hideAttributeBars(boolean)
    hideFloatingMarkers(boolean)
    hideGroupFrames(boolean)
    hideNameplateHealthbars(boolean)
    hideChat(boolean)
    hideCrosshair(boolean)
end

function ImmersiveHUDHider.ImmersiveHUDHiderToggler()
    if not ImmersiveHUDHider.savedVariables.ImmersiveHUDHider_isUIHidden then
        ImmersiveHUDHider.saveDefaultGameSettings()
        ImmersiveHUDHider.interfaceEventHandlerToggle()
        hideHUD(true)
        ImmersiveHUDHider.savedVariables.ImmersiveHUDHider_isUIHidden = true
    else
        ImmersiveHUDHider.interfaceEventHandlerToggle()
        hideHUD(false)
        ImmersiveHUDHider.savedVariables.ImmersiveHUDHider_isUIHidden = false
    end
end

function ImmersiveHUDHider:Initialize()

    ImmersiveHUDHider.savedVariables = ZO_SavedVars:NewAccountWide(
                                           "ImmersiveHUDHiderSavedVariables", ImmersiveHUDHider.version, nil, ImmersiveHUDHider.default
                                       )

    hideHUD(false)

    ImmersiveHUDHider.saveDefaultGameSettings()

    ImmersiveHUDHider.savedVariables.ImmersiveHUDHider_isUIHidden = false

    if ImmersiveHUDHider.savedVariables.ImmersiveHUDHider_isUIHidden == true then
        ImmersiveHUDHider.savedVariables.ImmersiveHUDHider_isUIHidden = false
    else
        ImmersiveHUDHider.savedVariables.ImmersiveHUDHider_isUIHidden = false
    end

    ImmersiveHUDHider.savedVariables.isInterfaceEventHandler = 0
    ImmersiveHUDHider.interfaceEventHandlerToggle()
    ImmersiveHUDHider.buildMenu()
    EVENT_MANAGER:UnregisterForEvent(ImmersiveHUDHider.name, EVENT_ADD_ON_LOADED)
end

function ImmersiveHUDHider.OnAddOnLoaded(event, addonName)
    if addonName == ImmersiveHUDHider.name then
        ImmersiveHUDHider:Initialize()
    end
end

EVENT_MANAGER:RegisterForEvent(ImmersiveHUDHider.name, EVENT_ADD_ON_LOADED, ImmersiveHUDHider.OnAddOnLoaded)

SLASH_COMMANDS["/immersivelyhideui"] = ImmersiveHUDHider.ImmersiveHUDHiderToggler

function ImmersiveHUDHider.buildMenu(addonName, version)
    local LAM = LibAddonMenu2

    local panelData = {
        type = "panel", name = "Customisable Immersive HUD Hider", displayName = "Customisable Immersive HUD Hider", author = "Alfthebigheaded",
        version = ImmersiveHUDHider.version, registerForRefresh = true, registerForDefaults = true
    }

    LAM:RegisterAddonPanel("Customisable Immersive HUD Hider", panelData)

    local optionsData = {
        {
            type = 'description',
            text = 'Choose which elements of the HUD you would like hidden on toggle. Once you have chosen your settings, please press the "Apply Settings" button at the bottom.'
        }, {type = "header", name = "Choose Which Elements To Hide"}, {type = "description", name = "desc"}, {
            type = "checkbox", name = "Hide Interactable Glow", tooltip = "Toggle outline around lootable objects", getFunc = function()
                return ImmersiveHUDHider.savedVariables.hideInteractableGlow
            end, setFunc = function(newValue)
                if (newValue) then
                    ImmersiveHUDHider.savedVariables.hideInteractableGlow = true
                else
                    ImmersiveHUDHider.savedVariables.hideInteractableGlow = false
                end
            end, width = "full", default = false
        }, {
            type = "checkbox", name = "Hide Target Glow", tooltip = "Togle outline around targetted enemy", getFunc = function()
                return ImmersiveHUDHider.savedVariables.hideTargetGlow
            end, setFunc = function(newValue)
                if (newValue) then
                    ImmersiveHUDHider.savedVariables.hideTargetGlow = true
                else
                    ImmersiveHUDHider.savedVariables.hideTargetGlow = false
                end
            end, width = "full", default = false
        }, {
            type = "checkbox", name = "Hide Quest Bestowers", tooltip = "Toggle in-world quest markers", getFunc = function()
                return ImmersiveHUDHider.savedVariables.hideQuestBestowerIndicator
            end, setFunc = function(newValue)
                if (newValue) then
                    ImmersiveHUDHider.savedVariables.hideQuestBestowerIndicator = true
                else
                    ImmersiveHUDHider.savedVariables.hideQuestBestowerIndicator = false
                end
            end, width = "full", default = false
        }, {
            type = "checkbox", name = "Hide Group Indicators", tooltip = "Toggle in-world group indicators", getFunc = function()
                return ImmersiveHUDHider.savedVariables.hideGroupIndicators
            end, setFunc = function(newValue)
                if (newValue) then
                    ImmersiveHUDHider.savedVariables.hideGroupIndicators = true
                else
                    ImmersiveHUDHider.savedVariables.hideGroupIndicators = false
                end
            end, width = "full", default = true
        }, {
            type = "checkbox", name = "Hide Follow Indicator", tooltip = "Toggle in-world follow indicators", getFunc = function()
                return ImmersiveHUDHider.savedVariables.hideFollowIndicator
            end, setFunc = function(newValue)
                if (newValue) then
                    ImmersiveHUDHider.savedVariables.hideFollowIndicator = true
                else
                    ImmersiveHUDHider.savedVariables.hideFollowIndicator = false
                end
            end, width = "full", default = true
        }, {
            type = "checkbox", name = "Hide Alliance Indicators", tooltip = "Toggle in-world alliance indicators", getFunc = function()
                return ImmersiveHUDHider.savedVariables.hideAllianceIndicators
            end, setFunc = function(newValue)
                if (newValue) then
                    ImmersiveHUDHider.savedVariables.hideAllianceIndicators = true
                else
                    ImmersiveHUDHider.savedVariables.hideAllianceIndicators = false
                end
            end, width = "full", default = true
        }, {
            type = "checkbox", name = "Hide Ressurrect Indicator", tooltip = "Toggle in-world ressurrection indicators", getFunc = function()
                return ImmersiveHUDHider.savedVariables.hideResurrectIndicator
            end, setFunc = function(newValue)
                if (newValue) then
                    ImmersiveHUDHider.savedVariables.hideResurrectIndicator = true
                else
                    ImmersiveHUDHider.savedVariables.hideResurrectIndicator = false
                end
            end, width = "full", default = false
        }, {
            type = "checkbox", name = "Hide Health Bars", tooltip = "Toggle above head health bars", getFunc = function()
                return ImmersiveHUDHider.savedVariables.hideAllHealthBars
            end, setFunc = function(newValue)
                if (newValue) then
                    ImmersiveHUDHider.savedVariables.hideAllHealthBars = true
                else
                    ImmersiveHUDHider.savedVariables.hideAllHealthBars = false
                end
            end, width = "full", default = false
        }, {
            type = "checkbox", name = "Hide Name Plates", tooltip = "Toggle above head nameplates", getFunc = function()
                return ImmersiveHUDHider.savedVariables.hideAllNamePlates
            end, setFunc = function(newValue)
                if (newValue) then
                    ImmersiveHUDHider.savedVariables.hideAllNamePlates = true
                else
                    ImmersiveHUDHider.savedVariables.hideAllNamePlates = false
                end
            end, width = "full", default = true
        }, {
            type = "checkbox", name = "Hide Chat Bubbles", tooltip = "Toggle chat bubbles for players", getFunc = function()
                return ImmersiveHUDHider.savedVariables.hideChatBubbles
            end, setFunc = function(newValue)
                if (newValue) then
                    ImmersiveHUDHider.savedVariables.hideChatBubbles = true
                else
                    ImmersiveHUDHider.savedVariables.hideChatBubbles = false
                end
            end, width = "full", default = true
        }, {
            type = "checkbox", name = "Hide Action Bar", tooltip = "Toggle action bar", getFunc = function()
                return ImmersiveHUDHider.savedVariables.hideActionBar
            end, setFunc = function(newValue)
                if (newValue) then
                    ImmersiveHUDHider.savedVariables.hideActionBar = true
                else
                    ImmersiveHUDHider.savedVariables.hideActionBar = false
                end
            end, width = "full", default = true
        }, {
            type = "checkbox", name = "Hide Attribute Bars", tooltip = "Toggle attribute bars", getFunc = function()
                return ImmersiveHUDHider.savedVariables.hideAttributeBar
            end, setFunc = function(newValue)
                if (newValue) then
                    ImmersiveHUDHider.savedVariables.hideAttributeBar = true
                else
                    ImmersiveHUDHider.savedVariables.hideAttributeBar = false
                end
            end, width = "full", default = true
        }, {
            type = "checkbox", name = "Hide Buffs", tooltip = "Toggle active player and target buffs", getFunc = function()
                return ImmersiveHUDHider.savedVariables.hideBuffs
            end, setFunc = function(newValue)
                if (newValue) then
                    ImmersiveHUDHider.savedVariables.hideBuffs = true
                else
                    ImmersiveHUDHider.savedVariables.hideBuffs = false
                end
            end, width = "full", default = true
        }, {
            type = "checkbox", name = "Hide Scrolling Combat Text", tooltip = "Toggle scrolling combat text", getFunc = function()
                return ImmersiveHUDHider.savedVariables.hideScrollingCombatText
            end, setFunc = function(newValue)
                if (newValue) then
                    ImmersiveHUDHider.savedVariables.hideScrollingCombatText = true
                else
                    ImmersiveHUDHider.savedVariables.hideScrollingCombatText = false
                end
            end, width = "full", default = true
        }, {
            type = "checkbox", name = "Hide Loot Log", tooltip = "Toggle loot log", getFunc = function()
                return ImmersiveHUDHider.savedVariables.hideLootLog
            end, setFunc = function(newValue)
                if (newValue) then
                    ImmersiveHUDHider.savedVariables.hideLootLog = true
                else
                    ImmersiveHUDHider.savedVariables.hideLootLog = false
                end
            end, width = "full", default = true
        }, {
            type = "checkbox", name = "Hide Compass", tooltip = "Toggle compass", getFunc = function()
                return ImmersiveHUDHider.savedVariables.hideCompass
            end, setFunc = function(newValue)
                if (newValue) then
                    ImmersiveHUDHider.savedVariables.hideCompass = true
                else
                    ImmersiveHUDHider.savedVariables.hideCompass = false
                end
            end, width = "full", default = true
        }, {
            type = "checkbox", name = "Hide Map", tooltip = "Toggle minimap", getFunc = function()
                return ImmersiveHUDHider.savedVariables.hideMap
            end, setFunc = function(newValue)
                if (newValue) then
                    ImmersiveHUDHider.savedVariables.hideMap = true
                else
                    ImmersiveHUDHider.savedVariables.hideMap = false
                end
            end, width = "full", default = true
        }, {
            type = "checkbox", name = "Hide Quest Tracker", tooltip = "Toggle quest tracker", getFunc = function()
                return ImmersiveHUDHider.savedVariables.hideQuestTracker
            end, setFunc = function(newValue)
                if (newValue) then
                    ImmersiveHUDHider.savedVariables.hideQuestTracker = true
                else
                    ImmersiveHUDHider.savedVariables.hideQuestTracker = false
                end
            end, width = "full", default = true
        }, {
            type = "checkbox", name = "Hide Boss Bar", tooltip = "Toggle boss bar", getFunc = function()
                return ImmersiveHUDHider.savedVariables.hideBossBar
            end, setFunc = function(newValue)
                if (newValue) then
                    ImmersiveHUDHider.savedVariables.hideBossBar = true
                else
                    ImmersiveHUDHider.savedVariables.hideBossBar = false
                end
            end, width = "full", default = true
        }, {
            type = "checkbox", name = "Hide Group Unit Frames", tooltip = "Toggle group & companion hud", getFunc = function()
                return ImmersiveHUDHider.savedVariables.hideGroupUnitFrames
            end, setFunc = function(newValue)
                if (newValue) then
                    ImmersiveHUDHider.savedVariables.hideGroupUnitFrames = true
                else
                    ImmersiveHUDHider.savedVariables.hideGroupUnitFrames = false
                end
            end, width = "full", default = true
        }, {
            type = "checkbox", name = "Hide Target Info", tooltip = "Toggle target information", getFunc = function()
                return ImmersiveHUDHider.savedVariables.hideTargetInfo
            end, setFunc = function(newValue)
                if (newValue) then
                    ImmersiveHUDHider.savedVariables.hideTargetInfo = true
                else
                    ImmersiveHUDHider.savedVariables.hideTargetInfo = false
                end
            end, width = "full", default = true
        }, {
            type = "dropdown", name = "Hide Chat", tooltip = "Choose an option to hide chat", choices = hideChatChoices, getFunc = function()
                return ImmersiveHUDHider.savedVariables.hideChat
            end, setFunc = function(newValue)
                ImmersiveHUDHider.savedVariables.hideChat = newValue
            end, width = "full", default = true
        }, {
            type = "dropdown", name = "Hide Crosshair", tooltip = "Choose an option to hide crosshair", choices = hideCrosshairChoices,
            getFunc = function()
                return ImmersiveHUDHider.savedVariables.hideCrosshair
            end, setFunc = function(newValue)
                ImmersiveHUDHider.savedVariables.hideCrosshair = newValue
            end, width = "full", default = true
        }, {
            type = "button", name = "Apply Settings", func = function()
                hideHUD(false)
                ImmersiveHUDHider.saveDefaultGameSettings()
                ReloadUI()
            end
        }

    }

    LAM:RegisterOptionControls("Customisable Immersive HUD Hider", optionsData)
end
